package com.huiju.estateagents.redpack.controller;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.estateagents.base.BaseController;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.common.StringUtils;
import com.huiju.estateagents.redpack.entity.TaOrgAccount;
import com.huiju.estateagents.redpack.service.ITaOrgAccountService;

/**
 * <p>
    * 系统账户表  前端控制器
    * </p>
 *
 * @author fxf
 * @since 2020-06-03
 */
@RestController
@RequestMapping("/channel")
public class TaOrgAccountController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaOrgAccountController.class);

    @Autowired
    public ITaOrgAccountService iTaOrgAccountService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/api/taOrgAccount",method= RequestMethod.GET)
    public ResponseBean taOrgAccountList(@RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                         @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                         @RequestParam(value ="miniAppName", required = false) String miniAppName){
        ResponseBean responseBean = new ResponseBean();
        try {
            //使用分页插件
		    IPage<TaOrgAccount> pg = new Page<>(pageNum, pageSize);
            QueryWrapper<TaOrgAccount> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq(!StringUtils.isEmpty(miniAppName), "miniapp_name", miniAppName);
            queryWrapper.orderByDesc("create_date");

            IPage<TaOrgAccount> result = iTaOrgAccountService.page(pg, queryWrapper);
            responseBean.addSuccess(result);
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taOrgAccountList -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 保存对象
     * @param taOrgAccount 实体对象
     * @return
     */
    @RequestMapping(value="/api/taOrgAccount",method= RequestMethod.POST)
    public ResponseBean taOrgAccountAdd(@RequestBody TaOrgAccount taOrgAccount){
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaOrgAccountService.save(taOrgAccount)){
                responseBean.addSuccess(taOrgAccount);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taOrgAccountAdd -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @ResponseBody
    @RequestMapping(value="/api/taOrgAccount/{id}", method= RequestMethod.DELETE)
    public ResponseBean taOrgAccountDelete(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            if(iTaOrgAccountService.removeById(id)){
                responseBean.addSuccess("success");
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taOrgAccountDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taOrgAccount 实体对象
     * @return
     */
    @RequestMapping(value="/api/taOrgAccount/{id}",method= RequestMethod.PUT)
    public ResponseBean taOrgAccountUpdate(@PathVariable Integer id,
                                        @RequestBody TaOrgAccount taOrgAccount){
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaOrgAccountService.updateById(taOrgAccount)){
                responseBean.addSuccess(taOrgAccount);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taOrgAccountUpdate -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/api/taOrgAccount/{id}",method= RequestMethod.GET)
    public ResponseBean taOrgAccountGet(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean.addSuccess(iTaOrgAccountService.getById(id));
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taOrgAccountDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 系统账户充值
     * @param id  实体ID
     */
    @RequestMapping(value="/taOrgAccount/recharge/{id}",method= RequestMethod.PUT)
    public ResponseBean taOrgAccountDetailedListGet(@PathVariable String id, @RequestBody TaOrgAccount taOrgAccount,
                                                    HttpServletRequest request){
        ResponseBean responseBean = new ResponseBean();
        try {
            Integer orgId = getOrgId(request);
            Integer userId = getUserId(request);
            iTaOrgAccountService.rechargeAccount(id, taOrgAccount, 26, 2149);
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taOrgAccountDetailedListDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }
}
