package com.huiju.estateagents.redpack.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.estateagents.base.BaseController;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.redpack.entity.TaRedPacket;
import com.huiju.estateagents.redpack.service.ITaRedPacketService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * <p>
    * 红包助力活动  前端控制器
    * </p>
 *
 * @author fxf
 * @since 2020-06-03
 */
@RestController
@RequestMapping("/")
public class TaRedPacketController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaRedPacketController.class);

    @Autowired
    public ITaRedPacketService iTaRedPacketService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/taRedPacket",method= RequestMethod.GET)
    public ResponseBean taRedPacketList(@RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                        @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize){
        ResponseBean responseBean = new ResponseBean();
        try {
            //使用分页插件
		    IPage<TaRedPacket> pg = new Page<>(pageNum, pageSize);
            QueryWrapper<TaRedPacket> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("create_date");

            IPage<TaRedPacket> result = iTaRedPacketService.page(pg, queryWrapper);
            responseBean.addSuccess(result);
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taRedPacketList -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 保存对象
     * @param taRedPacket 实体对象
     * @return
     */
    @RequestMapping(value="/taRedPacket",method= RequestMethod.POST)
    public ResponseBean taRedPacketAdd(@RequestBody TaRedPacket taRedPacket){
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaRedPacketService.save(taRedPacket)){
                responseBean.addSuccess(taRedPacket);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taRedPacketAdd -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @ResponseBody
    @RequestMapping(value="/taRedPacket/{id}", method= RequestMethod.DELETE)
    public ResponseBean taRedPacketDelete(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            if(iTaRedPacketService.removeById(id)){
                responseBean.addSuccess("success");
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taRedPacketDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taRedPacket 实体对象
     * @return
     */
    @RequestMapping(value="/taRedPacket/{id}",method= RequestMethod.PUT)
    public ResponseBean taRedPacketUpdate(@PathVariable Integer id,
                                        @RequestBody TaRedPacket taRedPacket){
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaRedPacketService.updateById(taRedPacket)){
                responseBean.addSuccess(taRedPacket);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taRedPacketUpdate -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/taRedPacket/{id}",method= RequestMethod.GET)
    public ResponseBean taRedPacketGet(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean.addSuccess(iTaRedPacketService.getById(id));
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taRedPacketDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }
}
