package com.huiju.estateagents.third.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.common.CommConstant;
import com.huiju.estateagents.common.MD5Utils;
import com.huiju.estateagents.drainage.entity.TaDrainage;
import com.huiju.estateagents.drainage.mapper.TaDrainageMapper;
import com.huiju.estateagents.redpack.entity.TaMpMiniapp;
import com.huiju.estateagents.redpack.entity.TaMpPerson;
import com.huiju.estateagents.redpack.entity.TaRedPacket;
import com.huiju.estateagents.redpack.entity.TaRedPacketPerson;
import com.huiju.estateagents.redpack.mapper.TaMpMiniappMapper;
import com.huiju.estateagents.redpack.mapper.TaMpPersonMapper;
import com.huiju.estateagents.redpack.mapper.TaRedPacketMapper;
import com.huiju.estateagents.redpack.mapper.TaRedPacketPersonMapper;
import com.huiju.estateagents.third.entity.TaThirdPartyMiniappConfig;
import com.huiju.estateagents.third.mapper.TaThirdPartyMiniappConfigMapper;
import com.huiju.estateagents.third.service.ITaThirdPartyMiniappConfigService;
import me.chanjar.weixin.mp.bean.result.WxMpUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

/**
 * <p>
 *   服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-10-17
 */
@Service
public class TaThirdPartyMiniappConfigServiceImpl extends ServiceImpl<TaThirdPartyMiniappConfigMapper, TaThirdPartyMiniappConfig> implements ITaThirdPartyMiniappConfigService {
	
	@Autowired
	private TaThirdPartyMiniappConfigMapper taThirdPartyMiniappConfigMapper;

	@Autowired
	private TaMpPersonMapper taMpPersonMapper;

	@Autowired
	private TaMpMiniappMapper taMpMiniappMapper;

	@Autowired
	private TaDrainageMapper taDrainageMapper;

	@Autowired
	private TaRedPacketMapper taRedPacketMapper;

	@Autowired
	private TaRedPacketPersonMapper taRedPacketPersonMapper;

	/**
	 * 校验是否合法第三方接口调用
	 *
	 * @param tokenHeader
	 * @param appidHeader
	 * @param nowHeader
	 * @return
	 */
	@Override
	public String checkToken(String tokenHeader, String appidHeader, String nowHeader) {
		//秘钥是否过期
		QueryWrapper<TaThirdPartyMiniappConfig> queryWrapper = new QueryWrapper<>();
		queryWrapper.eq("appid",appidHeader);
		TaThirdPartyMiniappConfig taThirdPartyMiniappConfig = taThirdPartyMiniappConfigMapper.selectOne(queryWrapper);
//		if (LocalDateTime.now().isAfter(taThirdPartyMiniappConfig.getExpirationDate())){
//			return "秘钥已过期，请重新申请";
//		}
		
		//判断nowHeader是十五分钟之内(15 * 60 * 1000)
		if (System.currentTimeMillis() - Long.valueOf(nowHeader) > (15 * 60 * 1000)){
			return "密钥超时，请在十五分钟内请求";
		}
		
		//重新加密对比
		StringBuilder stringBuilder = new StringBuilder();
		stringBuilder.append(taThirdPartyMiniappConfig.getAppid()).append(taThirdPartyMiniappConfig.getAppSecret()).append(nowHeader);
		String token = MD5Utils.md5(MD5Utils.md5(stringBuilder.toString()));
		if (!tokenHeader.equals(token)){
			return "秘钥出错请重新生成";
		}
		
		return "success";
	}

	/**
	 * 创建公众号人员并关联上小程序人员
	 *
	 * @param user
	 * @param xAppid
	 * @param miniAppid
	 * @param personId
	 */
	@Override
	public TaMpPerson newMpPerson(WxMpUser user,String mpAppid, String xAppid, String miniAppid, String personId) {
		//查询这个人员是否存在
		TaMpPerson mpPerson = taMpPersonMapper.selectOne(new QueryWrapper<TaMpPerson>().eq("mp_openid", user.getOpenId()));
		//不存在就新建
		if (null == mpPerson){
			//新增入公众号人员表
			TaMpPerson taMpPerson = new TaMpPerson();
			taMpPerson.setAppid(mpAppid);
			taMpPerson.setCity(user.getCity());
			taMpPerson.setCountry(user.getCountry());
			taMpPerson.setCreateDate(LocalDateTime.now());
			taMpPerson.setHeadimgurl(user.getHeadImgUrl());
			taMpPerson.setMpOpenid(user.getOpenId());
			taMpPerson.setMpPersonId(personId);
			taMpPerson.setNickname(user.getNickname());
			taMpPerson.setProvince(user.getProvince());
			taMpPerson.setSex(user.getSex());
			taMpPerson.setStatus(CommConstant.STATUS_NORMAL);
			taMpPersonMapper.insert(taMpPerson);
			mpPerson = taMpPerson;
		}

		//查看这个人是否绑定了小程序用户
		TaMpMiniapp taMpMiniapp = taMpMiniappMapper.selectOne(new QueryWrapper<TaMpMiniapp>().eq("mp_person_id", mpPerson.getMpPersonId()).eq("miniapp_person_id", personId));
		if (null == taMpMiniapp){
			taMpMiniapp = new TaMpMiniapp();
			taMpMiniapp.setMpPersonId(mpPerson.getMpPersonId());
			taMpMiniapp.setMiniappPersonId(personId);
			taMpMiniappMapper.insert(taMpMiniapp);
		}

		return mpPerson;
	}

	/**
	 * 参加活动生成参与记录等
	 *
	 * @param mpAppid 公众号appid
	 * @param mpPersonId 公众号人员id
	 * @param drainageId H5活动id
	 * @param orgId 小程序id
	 */
	@Override
	public ResponseBean participateActivity(String mpAppid, String mpPersonId, String drainageId, Integer orgId) {
		//第一步根据drainggeid查询出红包表对应的记录
		TaDrainage taDrainage = taDrainageMapper.selectById(drainageId);
		//第二步查询出这个H5对应的活动
		TaRedPacket taRedPacket = taRedPacketMapper.selectById(taDrainage.getActivityId());
		//判断这个活动的过期时间
		if (LocalDateTime.now().isAfter(taRedPacket.getEndDate())){
			return ResponseBean.error(taRedPacket.getActivityEndTip(),ResponseBean.ERROR_ILLEGAL_PARAMS);
		}
		if (LocalDateTime.now().isBefore(taRedPacket.getStartDate())){
			return ResponseBean.error("此活动未开始",ResponseBean.ERROR_ILLEGAL_PARAMS);
		}

		//查看这个人绑定的小程序用户
		TaMpMiniapp taMpMiniapp = taMpMiniappMapper.selectOne(new QueryWrapper<TaMpMiniapp>().eq("mp_person_id", mpPersonId));

		//查看此人是否参加过活动如果参加过直接返回
		QueryWrapper<TaRedPacketPerson> queryWrapper = new QueryWrapper<>();
		queryWrapper.eq("activity_id",taRedPacket.getActivityId());
		queryWrapper.eq("org_id",orgId);
		queryWrapper.eq("mp_person_id",mpPersonId);
		queryWrapper.eq("miniapp_person_id",taMpMiniapp.getMiniappPersonId());
		TaRedPacketPerson redPacketPerson = taRedPacketPersonMapper.selectOne(queryWrapper);
		if (null != redPacketPerson){
			return ResponseBean.success(redPacketPerson);
		}

		//参与活动
		TaRedPacketPerson taRedPacketPerson = new TaRedPacketPerson();
		taRedPacketPerson.setActivityId(taRedPacket.getActivityId());
		taRedPacketPerson.setCreateDate(LocalDateTime.now());
		taRedPacketPerson.setIsSignedUp(true);
		taRedPacketPerson.setMiniappPersonId(taMpMiniapp.getMiniappPersonId());
		taRedPacketPerson.setMpPersonId(mpPersonId);
		taRedPacketPerson.setOrgId(orgId);
		taRedPacketPersonMapper.insert(taRedPacketPerson);
		return ResponseBean.success(taRedPacketPerson);
	}
}
